@@ -1,10 +1,10 @@
-
 using System;
 using System.Collections;
 using System.Collections.Generic;
 using System.Diagnostics;
 using System.IO;
 using System.Xml;
+using System.Text.RegularExpressions;
 using Mono.Unix;
 
 namespace Tomboy
@@ -28,7 +28,7 @@ namespace Tomboy
 		int width, height;
 		int x, y;
 		bool open_on_startup;
-		
+
 		Dictionary<string, Tag> tags;
 
 		const int noPosition = -1;
@@ -39,40 +39,58 @@ namespace Tomboy
 			this.text = "";
 			x = noPosition;
 			y = noPosition;
-			
+
 			tags = new Dictionary<string, Tag> ();
-			
+
 			create_date = DateTime.MinValue;
 			change_date = DateTime.MinValue;
 		}
 
 		public string Uri
 		{
-			get { return uri; }
+			get {
+				return uri;
+			}
 		}
 
 		public string Title
 		{
-			get { return title; }
-			set { title = value; }
+			get {
+				return title;
+			}
+			set {
+				title = value;
+			}
 		}
 
 		public string Text
 		{
-			get { return text; }
-			set { text = value; }
+			get {
+				return text;
+			}
+			set {
+				text = value;
+			}
 		}
 
 		public DateTime CreateDate
 		{
-			get { return create_date; }
-			set { create_date = value; }
+			get {
+				return create_date;
+			}
+			set {
+				create_date = value;
+			}
 		}
 
 		public DateTime ChangeDate
 		{
-			get { return change_date; }
-			set { change_date = value; }
+			get {
+				return change_date;
+			}
+			set {
+				change_date = value;
+			}
 		}
 
 		// FIXME: the next five attributes don't belong here (the data
@@ -81,45 +99,71 @@ namespace Tomboy
 
 		public int CursorPosition
 		{
-			get { return cursor_pos; }
-			set { cursor_pos = value; }
+			get {
+				return cursor_pos;
+			}
+			set {
+				cursor_pos = value;
+			}
 		}
 
 		public int Width
 		{
-			get { return width; }
-			set { width = value; }
+			get {
+				return width;
+			}
+			set {
+				width = value;
+			}
 		}
 
 		public int Height
 		{
-			get { return height; }
-			set { height = value; }
+			get {
+				return height;
+			}
+			set {
+				height = value;
+			}
 		}
 
 		public int X
 		{
-			get { return x; }
-			set { x = value; }
+			get {
+				return x;
+			}
+			set {
+				x = value;
+			}
 		}
 
 		public int Y
 		{
-			get { return y; }
-			set { y = value; }
+			get {
+				return y;
+			}
+			set {
+				y = value;
+			}
 		}
-		
+
 		public Dictionary<string, Tag> Tags
 		{
-			get { return tags; }
+			get {
+				return tags;
+			}
 		}
-		
+
 		public bool IsOpenOnStartup
 		{
-			get { return open_on_startup; }
-			set { open_on_startup = value; }
+			get {
+				return open_on_startup;
+			}
+			set {
+				open_on_startup = value;
+			}
 		}
-		
+
 		public void SetPositionExtent (int x, int y, int width, int height)
 		{
 			Debug.Assert (x >= 0 && y >= 0);
@@ -166,12 +210,16 @@ namespace Tomboy
 
 		public NoteData Data
 		{
-			get { return data; }
+			get {
+				return data;
+			}
 		}
 
 		public NoteBuffer Buffer
 		{
-			get { return buffer; }
+			get {
+				return buffer;
+			}
 			set {
 				buffer = value;
 				buffer.Changed += BufferChanged;
@@ -212,7 +260,7 @@ namespace Tomboy
 		void SynchronizeText ()
 		{
 			if (TextInvalid () && buffer != null) {
-				data.Text = NoteBufferArchiver.Serialize (buffer); 
+				data.Text = NoteBufferArchiver.Serialize (buffer);
 			}
 		}
 
@@ -225,9 +273,9 @@ namespace Tomboy
 				buffer.Clear ();
 
 				// Load the stored xml text
-				NoteBufferArchiver.Deserialize (buffer, 
-								buffer.StartIter, 
-								data.Text);
+				NoteBufferArchiver.Deserialize (buffer,
+				                                buffer.StartIter,
+				                                data.Text);
 				buffer.Modified = false;
 
 				Gtk.TextIter cursor;
@@ -267,13 +315,14 @@ namespace Tomboy
 		}
 	}
 
-	public class Note 
+	public class Note
 	{
 		readonly NoteDataBufferSynchronizer data;
 
 		string filepath;
 
 		bool save_needed;
+		bool is_deleting;
 
 		NoteManager manager;
 		NoteWindow window;
@@ -282,6 +331,14 @@ namespace Tomboy
 
 		InterruptableTimeout save_timeout;
 
+		struct ChildWidgetData
+		{
+			public Gtk.TextChildAnchor anchor;
+			public Gtk.Widget widget;
+		};
+
+		Queue <ChildWidgetData> childWidgetQueue;
+
 		[System.Diagnostics.Conditional ("DEBUG_SAVE")]
 		static void DebugSave (string format, params object[] args)
 		{
@@ -293,26 +350,30 @@ namespace Tomboy
 			this.data = new NoteDataBufferSynchronizer (data);
 			this.filepath = filepath;
 			this.manager = manager;
-			
+
 			// Make sure each of the tags that NoteData found point to the
 			// instance of this note.
 			foreach (Tag tag in data.Tags.Values) {
 				AddTag (tag);
 			}
-			
+
 			save_timeout = new InterruptableTimeout ();
 			save_timeout.Timeout += SaveTimeout;
+
+			childWidgetQueue = new Queue <ChildWidgetData> ();
+			
+			is_deleting = false;
 		}
 
 		static string UrlFromPath (string filepath)
 		{
 			return "note://tomboy/" +
-				Path.GetFileNameWithoutExtension (filepath);
+			       Path.GetFileNameWithoutExtension (filepath);
 		}
 
 		public static Note CreateNewNote (string title,
-						  string filepath,
-						  NoteManager manager)
+		                                  string filepath,
+		                                  NoteManager manager)
 		{
 			NoteData data = new NoteData (UrlFromPath (filepath));
 			data.Title = title;
@@ -322,8 +383,8 @@ namespace Tomboy
 		}
 
 		public static Note CreateExistingNote (NoteData data,
-						       string filepath,
-						       NoteManager manager)
+		                                       string filepath,
+		                                       NoteManager manager)
 		{
 			if (data.CreateDate == DateTime.MinValue)
 				data.CreateDate = File.GetCreationTime (filepath);
@@ -334,8 +395,9 @@ namespace Tomboy
 
 		public void Delete ()
 		{
+			is_deleting = true;
 			save_timeout.Cancel ();
-			
+
 			// Remove the note from all the tags
 			foreach (Tag tag in Tags) {
 				RemoveTag (tag);
@@ -345,7 +407,7 @@ namespace Tomboy
 				window.Hide ();
 				window.Destroy ();
 			}
-			
+
 			// Remove note URI from GConf entry menu_pinned_notes
 			IsPinned = false;
 		}
@@ -359,8 +421,13 @@ namespace Tomboy
 			return note;
 		}
 
-		public void Save () 
+		public void Save ()
 		{
+			// Prevent any other condition forcing a save on the note
+			// if Delete has been called.
+			if (is_deleting)
+				return;
+			
 			// Do nothing if we don't need to save.  Avoids unneccessary saves
 			// e.g on forced quit when we call save for every note.
 			if (!save_needed)
@@ -369,7 +436,7 @@ namespace Tomboy
 			Logger.Log ("Saving '{0}'...", data.Data.Title);
 
 			NoteArchiver.Write (filepath, data.GetDataSynchronized ());
-			
+
 			if (Saved != null)
 				Saved (this);
 		}
@@ -430,27 +497,31 @@ namespace Tomboy
 			window.GetPosition (out cur_x, out cur_y);
 			window.GetSize (out cur_width, out cur_height);
 
-			if (data.Data.X == cur_x && 
-			    data.Data.Y == cur_y &&
-			    data.Data.Width == cur_width && 
-			    data.Data.Height == cur_height)
+			if (data.Data.X == cur_x &&
+			                data.Data.Y == cur_y &&
+			                data.Data.Width == cur_width &&
+			                data.Data.Height == cur_height)
 				return;
 
 			data.Data.SetPositionExtent (cur_x, cur_y, cur_width, cur_height);
-			
+
 			DebugSave ("WindowConfigureEvent queueing save");
 			QueueSave (false);
 		}
 
 		[GLib.ConnectBefore]
-		void WindowDestroyed (object sender, EventArgs args) 
+		void WindowDestroyed (object sender, EventArgs args)
 		{
 			window = null;
 		}
 
-		// Set a 4 second timeout to execute the save.  Possibly
-		// invalidate the text, which causes a re-serialize when the
-		// timeout is called...
+		/// <summary>
+		/// Set a 4 second timeout to execute the save.  Possibly
+		/// invalidate the text, which causes a re-serialize when the
+		/// timeout is called...
+		/// </summary>
+		/// <param name="content_changed">Indicates whether or not
+		/// to update the note's last change date</param>
 		public void QueueSave (bool content_changed)
 		{
 			DebugSave ("Got QueueSave");
@@ -477,14 +548,14 @@ namespace Tomboy
 				Logger.Log ("Error while saving: {0}", e);
 			}
 		}
-		
+
 		public void AddTag (Tag tag)
 		{
 			if (tag == null)
 				throw new ArgumentNullException ("Note.AddTag () called with a null tag.");
 
 			tag.AddNote (this);
-			
+
 			if (!data.Data.Tags.ContainsKey (tag.NormalizedName)) {
 				data.Data.Tags [tag.NormalizedName] = tag;
 
@@ -492,45 +563,94 @@ namespace Tomboy
 					TagAdded (this, tag);
 
 				DebugSave ("Tag added, queueing save");
-				QueueSave (false);
+				QueueSave (true);
 			}
 		}
-		
+
 		public void RemoveTag (Tag tag)
 		{
 			if (tag == null)
 				throw new ArgumentException ("Note.RemoveTag () called with a null tag.");
-			
+
 			if (!data.Data.Tags.ContainsKey (tag.NormalizedName))
 				return;
-			
+
 			if (TagRemoving != null)
 				TagRemoving (this, tag);
-			
+
 			data.Data.Tags.Remove (tag.NormalizedName);
 			tag.RemoveNote (this);
-			
+
 			if (TagRemoved != null)
 				TagRemoved (this, tag.NormalizedName);
 
 			DebugSave ("Tag removed, queueing save");
-			QueueSave (false);
+			QueueSave (true);
+		}
+		
+		public bool ContainsTag (Tag tag)
+		{
+			if (data.Data.Tags.ContainsKey (tag.NormalizedName) == true)
+				return true;
+			
+			return false;
+		}
+
+		public void AddChildWidget (Gtk.TextChildAnchor childAnchor, Gtk.Widget widget)
+		{
+			ChildWidgetData data = new ChildWidgetData ();
+			data.anchor = childAnchor;
+			data.widget = widget;
+
+			childWidgetQueue.Enqueue (data);
+
+			if (HasWindow)
+				ProcessChildWidgetQueue ();
+		}
+
+		private void ProcessChildWidgetQueue ()
+		{
+			// Insert widgets in the childWidgetQueue into the NoteEditor
+			if (!HasWindow)
+				return; // can't do anything without a window
+
+			foreach (ChildWidgetData data in childWidgetQueue) {
+				data.widget.Show();
+				Window.Editor.AddChildAtAnchor (data.widget, data.anchor);
+			}
+
+			childWidgetQueue.Clear ();
 		}
 
 		public string Uri
 		{
-			get { return data.Data.Uri; }
+			get {
+				return data.Data.Uri;
+			}
 		}
 
-		public string FilePath 
+		public string Id
 		{
-			get { return filepath; }
-			set { filepath = value; }
+			get {
+				return data.Data.Uri.Replace ("note://tomboy/","");        // TODO: Store on Note instantiation
+			}
 		}
 
-		public string Title 
+		public string FilePath
 		{
-			get { return data.Data.Title; }
+			get {
+				return filepath;
+			}
+			set {
+				filepath = value;
+			}
+		}
+
+		public string Title
+		{
+			get {
+				return data.Data.Title;
+			}
 			set {
 				if (data.Data.Title != value) {
 					if (window != null)
@@ -541,29 +661,154 @@ namespace Tomboy
 
 					if (Renamed != null)
 						Renamed (this, old_title);
+
+					QueueSave (true); // TODO: Right place for this?
 				}
 			}
 		}
 
-		public string XmlContent 
+		public void RenameWithoutLinkUpdate (string newTitle)
 		{
-			get { return data.Text; }
-			set { 
-				if (buffer != null)
-					buffer.SetText (XmlDecoder.Decode (value));
-				else
-					data.Text = value; 
+			if (data.Data.Title != newTitle) {
+				if (window != null)
+					window.Title = newTitle;
+
+				data.Data.Title = newTitle;
+
+				// HACK:
+				if (Renamed != null)
+					Renamed (this, newTitle);
+
+				QueueSave (true); // TODO: Right place for this?
 			}
 		}
 
+		public string XmlContent
+		{
+			get {
+				return data.Text;
+			}
+			set {
+				if (buffer != null) {
+					buffer.SetText("");
+					NoteBufferArchiver.Deserialize (buffer, value);
+				} else
+					data.Text = value;
+			}
+		}
+
+		/// <summary>
+		/// Return the complete contents of this note's .note XML file
+		/// In case of any error, null is returned.
+		/// </summary>
+		public string GetCompleteNoteXml ()
+		{
+			if (!File.Exists (filepath))
+				return null;
+
+			// Make sure file contents are up to date
+			save_needed = true; // HACK: Catches newly created notes
+			Save ();
+
+			StreamReader reader = null;
+			try {
+				reader = new StreamReader (filepath);
+				return reader.ReadToEnd ();
+			} catch (Exception e) {
+				Logger.Error ("Error received while attempting to read " +
+				              filepath + ": " + e.Message);
+				return null;
+			} finally {
+				if (reader != null)
+					reader.Close ();
+			}
+		}
+
+		// Reload note data from a complete note XML string
+		// Should referesh note window, too
+		public void LoadForeignNoteXml (string foreignNoteXml)
+		{
+			if (foreignNoteXml == null)
+				throw new ArgumentNullException ("foreignNoteXml");
+
+			// Arguments to this method cannot be trusted.  If this method
+			// were to throw an XmlException in the middle of processing,
+			// a note could be damaged.  Therefore, we check for parseability
+			// ahead of time, and throw early.
+			XmlDocument xmlDoc = new XmlDocument ();
+			// This will throw an XmlException if foreignNoteXml is not parseable
+			xmlDoc.LoadXml (foreignNoteXml);
+			xmlDoc = null;
+
+			StringReader reader = new StringReader (foreignNoteXml);
+			XmlTextReader xml = new XmlTextReader (reader);
+			xml.Namespaces = false;
+
+			// Remove tags now, since a note with no tags has
+			// no "tags" element in the XML
+			foreach (Tag tag in Tags)
+			RemoveTag (tag);
+
+			while (xml.Read ()) {
+				switch (xml.NodeType) {
+				case XmlNodeType.Element:
+					switch (xml.Name) {
+					case "title":
+						Title = xml.ReadString ();
+						break;
+					case "text":
+						XmlContent = xml.ReadInnerXml ();
+						break;
+					case "last-change-date":
+						data.Data.ChangeDate =
+						        XmlConvert.ToDateTime (xml.ReadString (), NoteArchiver.DATE_TIME_FORMAT);
+						break;
+					case "create-date":
+						data.Data.CreateDate =
+						        XmlConvert.ToDateTime (xml.ReadString (), NoteArchiver.DATE_TIME_FORMAT);
+						break;
+					case "tags":
+						XmlDocument doc = new XmlDocument ();
+						List<string> tag_strings = ParseTags (doc.ReadNode (xml.ReadSubtree ()));
+						foreach (string tag_str in tag_strings) {
+							Tag tag = TagManager.GetOrCreateTag (tag_str);
+							AddTag (tag);
+						}
+						break;
+					case "open-on-startup":
+						IsOpenOnStartup = bool.Parse (xml.ReadString ());
+						break;
+					}
+					break;
+				}
+			}
+
+			xml.Close ();
+
+			// TODO: Any reason to queue a save here?  Maybe not for sync but for others?
+		}
+
+		// TODO: CODE DUPLICATION SUCKS
+		List<string> ParseTags (XmlNode tagNodes)
+		{
+			List<string> tags = new List<string> ();
+
+			foreach (XmlNode node in tagNodes.SelectNodes ("//tag")) {
+				string tag = node.InnerText;
+				tags.Add (tag);
+			}
+
+			return tags;
+		}
+
 		public string TextContent
 		{
 			get {
 				if (buffer != null)
-					return buffer.GetSlice (buffer.StartIter, 
-								buffer.EndIter, 
-								false /* hidden_chars */);
-				else 
+					return buffer.GetSlice (buffer.StartIter,
+					buffer.EndIter,
+					false /* hidden_chars */);
+				else
 					return XmlDecoder.Decode (XmlContent);
 			}
 			set {
@@ -572,47 +817,57 @@ namespace Tomboy
 				else
 					Logger.Log ("Setting text content for closed notes not supported");
 			}
-				
+
 		}
 
 		public NoteData Data
 		{
-			get { return data.GetDataSynchronized (); }
+			get {
+				return data.GetDataSynchronized ();
+			}
 		}
 
-		public DateTime CreateDate 
+		public DateTime CreateDate
 		{
-			get { return data.Data.CreateDate; }
+			get {
+				return data.Data.CreateDate;
+			}
 		}
 
-		public DateTime ChangeDate 
+		public DateTime ChangeDate
 		{
-			get { return data.Data.ChangeDate; }
+			get {
+				return data.Data.ChangeDate;
+			}
 		}
 
 		public NoteManager Manager
 		{
-			get { return manager; }
-			set { manager = value; }
+			get {
+				return manager;
+			}
+			set {
+				manager = value;
+			}
 		}
 
 		public NoteTagTable TagTable
 		{
 			get {
 				if (tag_table == null) {
-#if FIXED_GTKSPELL
+					#if FIXED_GTKSPELL
 					// NOTE: Sharing the same TagTable means
 					// that formatting is duplicated between
 					// buffers.
 					tag_table = NoteTagTable.Instance;
-#else
+					#else
 					// NOTE: GtkSpell chokes on shared
 					// TagTables because it blindly tries to
 					// create a new "gtkspell-misspelling"
 					// tag, which fails if one already
 					// exists in the table.
 					tag_table = new NoteTagTable ();
-#endif
+					#endif
 				}
 				return tag_table;
 			}
@@ -620,17 +875,19 @@ namespace Tomboy
 
 		public bool HasBuffer
 		{
-			get { return null != buffer; }
+			get {
+				return null != buffer;
+			}
 		}
 
 		public NoteBuffer Buffer
 		{
 			get {
 				if (buffer == null) {
-					Logger.Log ("Creating Buffer for '{0}'...", 
-						    data.Data.Title);
+					Logger.Log ("Creating Buffer for '{0}'...",
+					data.Data.Title);
 
-					buffer = new NoteBuffer (TagTable);
+					buffer = new NoteBuffer (TagTable, this);
 					data.Buffer = buffer;
 
 					// Listen for further changed signals
@@ -645,20 +902,22 @@ namespace Tomboy
 
 		public bool HasWindow
 		{
-			get { return null != window; }
+			get {
+				return null != window;
+			}
 		}
 
-		public NoteWindow Window 
+		public NoteWindow Window
 		{
 			get {
 				if (window == null) {
 					window = new NoteWindow (this);
 					window.Destroyed += WindowDestroyed;
 					window.ConfigureEvent += WindowConfigureEvent;
-					
+
 					if (data.Data.HasExtent ())
-						window.SetDefaultSize (data.Data.Width, 
-								       data.Data.Height);
+						window.SetDefaultSize (data.Data.Width,
+						                       data.Data.Height);
 
 					if (data.Data.HasPosition ())
 						window.Move (data.Data.X, data.Data.Y);
@@ -667,48 +926,58 @@ namespace Tomboy
 					// OnRealized causes segfaults.
 					if (Opened != null)
 						Opened (this, new EventArgs ());
+
+					// Add any child widgets if any exist now that
+					// the window is showing.
+					ProcessChildWidgetQueue ();
 				}
-				return window; 
+				return window;
 			}
 		}
 
-		public bool IsSpecial 
+		public bool IsSpecial
 		{
-			get { return NoteManager.StartNoteUri == data.Data.Uri; }
+			get {
+				return NoteManager.StartNoteUri == data.Data.Uri;
+			}
 		}
 
-		public bool IsNew 
+		public bool IsNew
 		{
-			get { 
+			get {
 				// Note is new if created in the last 24 hours.
 				return data.Data.CreateDate > DateTime.Now.AddHours (-24);
 			}
 		}
 
-		public bool IsLoaded 
+		public bool IsLoaded
 		{
-			get { return buffer != null; }
+			get {
+				return buffer != null;
+			}
 		}
 
-		public bool IsOpened 
+		public bool IsOpened
 		{
-			get { return window != null; }
+			get {
+				return window != null;
+			}
 		}
 
 		public bool IsPinned
 		{
 			get {
 				string pinned_uris = (string)
-					Preferences.Get (Preferences.MENU_PINNED_NOTES);
+				Preferences.Get (Preferences.MENU_PINNED_NOTES);
 				return pinned_uris.IndexOf (Uri) > -1;
 			}
 			set {
 				string new_pinned = "";
 				string old_pinned = (string)
-					Preferences.Get (Preferences.MENU_PINNED_NOTES);
+				                    Preferences.Get (Preferences.MENU_PINNED_NOTES);
 				bool pinned = old_pinned.IndexOf (Uri) > -1;
 
-				if (value == pinned) 
+				if (value == pinned)
 					return;
 
 				if (value) {
@@ -725,10 +994,12 @@ namespace Tomboy
 				Preferences.Set (Preferences.MENU_PINNED_NOTES, new_pinned);
 			}
 		}
-		
+
 		public bool IsOpenOnStartup
 		{
-			get { return Data.IsOpenOnStartup; }
+			get {
+				return Data.IsOpenOnStartup;
+			}
 			set {
 				if (Data.IsOpenOnStartup != value) {
 					Data.IsOpenOnStartup = value;
@@ -736,10 +1007,12 @@ namespace Tomboy
 				}
 			}
 		}
-		
+
 		public List<Tag> Tags
 		{
-			get { return new List<Tag> (data.Data.Tags.Values); }
+			get {
+				return new List<Tag> (data.Data.Tags.Values);
+			}
 		}
 
 		public event EventHandler Opened;
@@ -756,7 +1029,7 @@ namespace Tomboy
 	public class NoteArchiver
 	{
 		public const string CURRENT_VERSION = "0.2";
-		
+
 		public const string DATE_TIME_FORMAT = "yyyy-MM-ddTHH:mm:ss.fffffffzzz";
 
 		static NoteArchiver instance = null;
@@ -790,13 +1063,13 @@ namespace Tomboy
 			return Instance.ReadFile (read_file, uri);
 		}
 
-		public virtual NoteData ReadFile (string read_file, string uri) 
+		public virtual NoteData ReadFile (string read_file, string uri)
 		{
 			NoteData note = new NoteData (uri);
 			string version = "";
 
-			StreamReader reader = new StreamReader (read_file, 
-								System.Text.Encoding.UTF8);
+			StreamReader reader = new StreamReader (read_file,
+			                                        System.Text.Encoding.UTF8);
 			XmlTextReader xml = new XmlTextReader (reader);
 			xml.Namespaces = false;
 
@@ -816,12 +1089,12 @@ namespace Tomboy
 						note.Text = xml.ReadInnerXml ();
 						break;
 					case "last-change-date":
-						note.ChangeDate = 
-							XmlConvert.ToDateTime (xml.ReadString (), DATE_TIME_FORMAT);
+						note.ChangeDate =
+						        XmlConvert.ToDateTime (xml.ReadString (), DATE_TIME_FORMAT);
 						break;
 					case "create-date":
-						note.CreateDate = 
-							XmlConvert.ToDateTime (xml.ReadString (), DATE_TIME_FORMAT);
+						note.CreateDate =
+						        XmlConvert.ToDateTime (xml.ReadString (), DATE_TIME_FORMAT);
 						break;
 					case "cursor-position":
 						note.CursorPosition = int.Parse (xml.ReadString ());
@@ -871,7 +1144,7 @@ namespace Tomboy
 			Instance.WriteFile (write_file, note);
 		}
 
-		public virtual void WriteFile (string write_file, NoteData note) 
+		public virtual void WriteFile (string write_file, NoteData note)
 		{
 			string tmp_file = write_file + ".tmp";
 
@@ -916,18 +1189,18 @@ namespace Tomboy
 
 			xml.WriteStartDocument ();
 			xml.WriteStartElement (null, "note", "http://beatniksoftware.com/tomboy");
-			xml.WriteAttributeString(null, 
-						 "version", 
-						 null, 
-						 CURRENT_VERSION);
-			xml.WriteAttributeString("xmlns", 
-						 "link", 
-						 null, 
-						 "http://beatniksoftware.com/tomboy/link");
-			xml.WriteAttributeString("xmlns", 
-						 "size", 
-						 null, 
-						 "http://beatniksoftware.com/tomboy/size");
+			xml.WriteAttributeString(null,
+			                         "version",
+			                         null,
+			                         CURRENT_VERSION);
+			xml.WriteAttributeString("xmlns",
+			                         "link",
+			                         null,
+			                         "http://beatniksoftware.com/tomboy/link");
+			xml.WriteAttributeString("xmlns",
+			                         "size",
+			                         null,
+			                         "http://beatniksoftware.com/tomboy/size");
 
 			xml.WriteStartElement (null, "title", null);
 			xml.WriteString (note.Title);
@@ -941,13 +1214,13 @@ namespace Tomboy
 
 			xml.WriteStartElement (null, "last-change-date", null);
 			xml.WriteString (
-						XmlConvert.ToString (note.ChangeDate, DATE_TIME_FORMAT));
+			        XmlConvert.ToString (note.ChangeDate, DATE_TIME_FORMAT));
 			xml.WriteEndElement ();
 
 			if (note.CreateDate != DateTime.MinValue) {
 				xml.WriteStartElement (null, "create-date", null);
 				xml.WriteString (
-						XmlConvert.ToString (note.CreateDate, DATE_TIME_FORMAT));
+				        XmlConvert.ToString (note.CreateDate, DATE_TIME_FORMAT));
 				xml.WriteEndElement ();
 			}
 
@@ -970,7 +1243,7 @@ namespace Tomboy
 			xml.WriteStartElement (null, "y", null);
 			xml.WriteString (note.Y.ToString ());
 			xml.WriteEndElement ();
-			
+
 			if (note.Tags.Count > 0) {
 				xml.WriteStartElement (null, "tags", null);
 				foreach (Tag tag in note.Tags.Values) {
@@ -980,7 +1253,7 @@ namespace Tomboy
 				}
 				xml.WriteEndElement ();
 			}
-			
+
 			xml.WriteStartElement (null, "open-on-startup", null);
 			xml.WriteString (note.IsOpenOnStartup.ToString ());
 			xml.WriteEndElement ();
@@ -988,36 +1261,78 @@ namespace Tomboy
 			xml.WriteEndElement (); // Note
 			xml.WriteEndDocument ();
 		}
-		
+
 		// <summary>
 		// Parse the tags from the <tags> element
 		// </summary>
 		List<string> ParseTags (XmlNode tagNodes)
 		{
 			List<string> tags = new List<string> ();
-			
+
 			foreach (XmlNode node in tagNodes.SelectNodes ("//tag")) {
 				string tag = node.InnerText;
 				tags.Add (tag);
 			}
-			
+
 			return tags;
 		}
+
+		public virtual string GetRenamedNoteXml (string noteXml, string oldTitle, string newTitle)
+		{
+			string updatedXml;
+
+			// Replace occurences of oldTitle with newTitle in noteXml
+			string titleTagPattern =
+			        string.Format ("<title>{0}</title>", oldTitle);
+			string titleTagReplacement =
+			        string.Format ("<title>{0}</title>", newTitle);
+			updatedXml = Regex.Replace (noteXml, titleTagPattern, titleTagReplacement);
+
+			string titleContentPattern =
+			        string.Format ("<note-content([^>]*)>\\s*{0}", oldTitle);
+			string titleContentReplacement =
+			        string.Format ("<note-content$1>{0}", newTitle);
+			updatedXml = Regex.Replace (updatedXml, titleContentPattern, titleContentReplacement);
+
+			return updatedXml;
+		}
+
+		public virtual string GetTitleFromNoteXml (string noteXml)
+		{
+			if (noteXml != null && noteXml.Length > 0) {
+				XmlTextReader xml = new XmlTextReader (new StringReader (noteXml));
+				xml.Namespaces = false;
+
+				while (xml.Read ()) {
+					switch (xml.NodeType) {
+					case XmlNodeType.Element:
+						switch (xml.Name) {
+						case "title":
+							return xml.ReadString ();
+							break;
+						}
+						break;
+					}
+				}
+			}
+
+			return null;
+		}
 	}
 
 	public class NoteUtils
 	{
-		public static void ShowDeletionDialog (Note note, Gtk.Window parent) 
+		public static void ShowDeletionDialog (Note note, Gtk.Window parent)
 		{
-			HIGMessageDialog dialog = 
-				new HIGMessageDialog (
-					parent,
-					Gtk.DialogFlags.DestroyWithParent,
-					Gtk.MessageType.Question,
-					Gtk.ButtonsType.None,
-					Catalog.GetString ("Really delete this note?"),
-					Catalog.GetString ("If you delete a note it is " +
-							   "permanently lost."));
+			HIGMessageDialog dialog =
+			        new HIGMessageDialog (
+			        parent,
+			        Gtk.DialogFlags.DestroyWithParent,
+			        Gtk.MessageType.Question,
+			        Gtk.ButtonsType.None,
+			        Catalog.GetString ("Really delete this note?"),
+			        Catalog.GetString ("If you delete a note it is " +
+			                           "permanently lost."));
 
 			Gtk.Button button;
 
